/*
 * B-tree type functions
 *
 * Copyright (c) 2011-2012, Joachim Metz <jbmetz@users.sourceforge.net>
 *
 * Refer to AUTHORS for acknowledgements.
 *
 * This software is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this software.  If not, see <http://www.gnu.org/licenses/>.
 */

#if !defined( _LIBVSHADOW_BTREE_TYPE_H )
#define _LIBVSHADOW_BTREE_TYPE_H

#include <common.h>
#include <types.h>

#include "libvshadow_array_type.h"
#include "libvshadow_libcerror.h"
#include "libvshadow_list_type.h"
#include "libvshadow_tree_type.h"

#if defined( __cplusplus )
extern "C" {
#endif

/* The tree node comparison definitions
 */
enum LIBVSHADOW_BTREE_NODE_COMPARE_DEFINITIONS
{
	/* The first value is less than the second value
	 */
        LIBVSHADOW_BTREE_NODE_COMPARE_LESS,

	/* The first and second values are equal
	 */
        LIBVSHADOW_BTREE_NODE_COMPARE_EQUAL,

	/* The first value is greater than the second value
	 */
        LIBVSHADOW_BTREE_NODE_COMPARE_GREATER
};

/* The B-tree type uses the list and tree type
 * The value of a tree node contains a list
 */
typedef struct libvshadow_btree libvshadow_btree_t;

struct libvshadow_btree
{
	/* The values array
	 */
	libvshadow_array_t *values_array;

	/* The root node
	 */
	libvshadow_tree_node_t *root_node;

	/* The maximum number of values
	 */
	int maximum_number_of_values;
};

int libvshadow_btree_initialize(
     libvshadow_btree_t **tree,
     int maximum_number_of_sub_nodes,
     libcerror_error_t **error );

int libvshadow_btree_free(
     libvshadow_btree_t **tree,
     int (*value_free_function)(
            intptr_t **value,
            libcerror_error_t **error ),
     libcerror_error_t **error );

int libvshadow_btree_free_values_list(
     libvshadow_list_t **values_list,
     libcerror_error_t **error );

/* TODO clone function */

int libvshadow_btree_get_number_of_values(
     libvshadow_btree_t *tree,
     int *number_of_values,
     libcerror_error_t **error );

int libvshadow_btree_get_value_by_index(
     libvshadow_btree_t *tree,
     int value_index,
     intptr_t **value,
     libcerror_error_t **error );

int libvshadow_btree_node_get_upper_node_by_value(
     libvshadow_tree_node_t *node,
     intptr_t *value,
     int (*value_compare_function)(
            intptr_t *first_value,
            intptr_t *second_value,
            libcerror_error_t **error ),
     libvshadow_tree_node_t **upper_node,
     libvshadow_list_element_t **upper_list_element,
     libcerror_error_t **error );

int libvshadow_btree_node_append_value(
     libvshadow_tree_node_t *node,
     intptr_t *value,
     libcerror_error_t **error );

int libvshadow_btree_node_insert_value(
     libvshadow_tree_node_t *node,
     intptr_t *value,
     int (*value_compare_function)(
            intptr_t *first_value,
            intptr_t *second_value,
            libcerror_error_t **error ),
     libcerror_error_t **error );

int libvshadow_btree_insert_value(
     libvshadow_btree_t *tree,
     int *value_index,
     intptr_t *value,
     int (*value_compare_function)(
            intptr_t *first_value,
            intptr_t *second_value,
            libcerror_error_t **error ),
     libvshadow_list_element_t **existing_list_element,
     libcerror_error_t **error );

#if defined( __cplusplus )
}
#endif

#endif

