/*
 * The Volume Shadow Snapshot (VSS) volume definition
 *
 * Copyright (C) 2011-2012, Joachim Metz <jbmetz@users.sourceforge.net>
 *
 * Refer to AUTHORS for acknowledgements.
 *
 * This software is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this software.  If not, see <http://www.gnu.org/licenses/>.
 */

#if !defined( _VSHADOW_VOLUME_H )
#define _VSHADOW_VOLUME_H

#include <common.h>
#include <types.h>

#if defined( __cplusplus )
extern "C" {
#endif

typedef struct vshadow_volume_header vshadow_volume_header_t;

struct vshadow_volume_header
{
	/* The identifier
	 * Consists of 16 bytes
	 * Contains a GUID
	 */
	uint8_t identifier[ 16 ];

	/* The version
	 * Consists of 4 bytes
	 */
	uint8_t version[ 4 ];

	/* The record type
	 * Consists of 4 bytes
	 */
	uint8_t record_type[ 4 ];

	/* The (current) offset
	 * Consists of 8 bytes
	 */
	uint8_t offset[ 8 ];

	/* Unknown
	 * Consists of 8 bytes
	 */
	uint8_t unknown1[ 8 ];

	/* Unknown
	 * Consists of 8 bytes
	 */
	uint8_t unknown2[ 8 ];

	/* The catalog offset
	 * Consists of 8 bytes
	 */
	uint8_t catalog_offset[ 8 ];

	/* Unknown
	 * Consists of 8 bytes
	 */
	uint8_t unknown3[ 8 ];

	/* The volume identifier
	 * Consists of 16 bytes
	 * Contains a GUID
	 */
	uint8_t volume_identifier[ 16 ];

	/* The store volume identifier
	 * Consists of 16 bytes
	 * Contains a GUID
	 */
	uint8_t store_volume_identifier[ 16 ];

	/* Unknown
	 * Consists of 416 bytes
	 */
	uint8_t unknown4[ 416 ];
};

#if defined( __cplusplus )
}
#endif

#endif

